@@ -58,13 +58,13 @@ test_that("NMA", {
 
   ## Orthognals
   expect_that(as.numeric(modes$U[,7] %*% modes$U[,7]),
-              equals(1))
+              equals(1, tolerance=1e-6))
   expect_that(as.numeric(modes$U[,7] %*% modes$U[,8]),
               equals(0, tolerance=1e-6))
   expect_that(all((round(c(modes$U[,7] %*% modes$U),6)==0)[-7]),
               equals(TRUE))
   
-  expect_that(all(modes$L[1:6]==0), equals(TRUE))
+  expect_that(all(round(c(modes$L[1:6]), 6)==0), equals(TRUE))
   
 
   ###################################################################
@@ -193,7 +193,9 @@ test_that("NMA", {
   ###################################################################
 
   mc <- list(ALA=500, SER=1000)
-  invisible(capture.output(modes <- nma(pdb, mass.custom=mc)))
+  suppressWarnings(
+    invisible(capture.output(modes <- nma(pdb, mass.custom=mc)))
+  )
 
   mass.expected <- c(500.000, 500.000, 500.000, 131.196, 129.180, 157.194)
   expect_that(modes$mass[9:14], equals(mass.expected, tolerance=1e-6))
@@ -216,7 +218,7 @@ test_that("NMA", {
   #
   ###################################################################
 
-  sele <- atom.select(pdb, "//A////CA/", verbose=FALSE)
+  sele <- atom.select(pdb, chain="A", elety="CA")
   xyz <- pdb$xyz[sele$xyz]
 
   i <- 2; j <- 5;
@@ -240,5 +242,23 @@ test_that("NMA", {
               equals(hessian2[atom2xyz(j), atom2xyz(i)], tolerance=1e-6))
   
    
+  ###################################################################
+  #
+  # Test extracting effective hessian with 'outmodes' argument 
+  #
+  ###################################################################
+
+  out.inds <- atom.select(pdb, 'calpha', resno=5:124)
+  invisible(capture.output(modes <- nma(pdb, outmodes = out.inds)))
+
+  U7 <- c(0.022538557,  0.016313285, -0.007462564,  
+          0.024520684, -0.005453823, -0.015629021)
+  nowU7 <- head(modes$U[,7])
+  expect_that(nowU7 * mysign(U7, nowU7), equals(U7, tolerance=1e-6))
+
+  eival <- c(0.013868, 0.015537, 0.024559, 0.030261, 0.039459, 0.043531)
+  nowEival <- modes$L[7:12]
+  expect_that(nowEival, equals(eival, tolerance=1e-6))
+   
 }
           )
